@@ -21,8 +21,11 @@ import shutil
 import sys
 import tempfile
 
+import ZConfig
+import zpkgsetup
 import zpkgtools
 
+from zpkgsetup import cfgparser
 from zpkgsetup import loggingapi as logging
 from zpkgsetup import package
 from zpkgsetup import publication
@@ -37,12 +40,13 @@ from zpkgtools import loader
 from zpkgtools import runlog
 
 
-zpkgsetup_dir = os.path.join(
-    os.path.dirname(os.path.dirname(os.path.abspath(__file__))),
-    "zpkgsetup")
+def package_resource(package):
+    dir = os.path.abspath(package.__path__[0])
+    return urlutils.file_url(dir)
 
 DEFAULT_SUPPORT_PACKAGES = [
-    ("zpkgsetup", urlutils.file_url(zpkgsetup_dir)),
+    ("ZConfig",   package_resource(ZConfig)),
+    ("zpkgsetup", package_resource(zpkgsetup)),
     ]
 
 
@@ -63,19 +67,20 @@ class Application:
         location_maps = []
         for map in options.location_maps:
             if os.path.isfile(map):
-                map = os.path.abspath(map)
-                map = "file://" + urlutils.pathname2url(map)
+                map = urlutils.file_url(os.path.abspath(map))
             location_maps.append(map)
         cf.location_maps.extend(location_maps)
         path = options.configfile
         if path is None:
             path = config.defaultConfigurationPath()
-            if os.path.exists(path):
-                self.logger.debug("loading configuration file %s", path)
-                cf.loadPath(path)
-        elif path:
+            if not os.path.exists(path):
+                path = None
+        if path:
             self.logger.debug("loading configuration file %s", path)
-            cf.loadPath(path)
+            try:
+                cf.loadPath(path)
+            except cfgparser.ConfigurationError, e:
+                self.error(str(e))
         cf.finalize()
         self.locations = cf.locations
         if cf.collect_dependencies:
@@ -83,9 +88,25 @@ class Application:
         if cf.application:
             self.options.application = True
 
+        self.exclude_packages = sets.Set(cf.exclude_packages)
+
         # XXX Hack: This should be part of BuilderApplication
         if options.include_support_code is None:
             options.include_support_code = cf.include_support_code
+        if options.resource is None:
+            if not cf.default_collection:
+                self.error("no resource to package specified")
+            options.resource = cf.default_collection
+        if not options.release_name:
+            options.release_name = cf.release_name
+
+        pkgs = sets.Set()
+        for pkgname, location in options.support_packages:
+            pkgs.add(pkgname)
+        for pkgname, location in cf.support_packages.iteritems():
+            if pkgname not in pkgs:
+                options.support_packages.append((pkgname, location))
+                pkgs.add(pkgname)
 
     def error(self, message, rc=1):
         self.logger.critical(message)
@@ -99,8 +120,6 @@ class BuilderApplication(Application):
         self.manifests = []
         self.ip = None
         self.resource = options.resource
-        if not options.release_name:
-            options.release_name = self.resource
         # Create a new directory for all temporary files to go in:
         self.tmpdir = tempfile.mkdtemp(prefix=options.program + "-")
         self.old_tmpdir = tempfile.tempdir
@@ -115,18 +134,25 @@ class BuilderApplication(Application):
             self.error("unknown resource: %s" % self.resource)
         self.resource_url = self.locations[self.resource]
         #
-        release_name = self.options.release_name
+        release_name = options.release_name or self.resource
         self.target_name = "%s-%s" % (release_name, self.options.version)
         self.target_file = self.target_name + ".tgz"
-        self.destination = os.path.join(self.tmpdir, self.target_name)
+        if self.options.tree_only:
+            self.destination = os.path.abspath(self.target_name)
+        else:
+            self.destination = os.path.join(self.tmpdir, self.target_name)
         os.mkdir(self.destination)
-        self.support_packages = DEFAULT_SUPPORT_PACKAGES[:]
-        self.support_packages.extend(
-            [(pkg, None) for pkg in options.support_packages])
-        self.exclude_packages = sets.Set()
+        pkgs = sets.Set()
+        self.support_packages = []
+        for pkgname, location in options.support_packages:
+            if pkgname not in pkgs:
+                self.support_packages.append((pkgname, location))
+        for pkgname, location in DEFAULT_SUPPORT_PACKAGES:
+            if pkgname not in pkgs:
+                self.support_packages.append((pkgname, location))
+                pkgs.add(pkgname)
         for pkg in options.exclude_packages:
-            if pkg not in self.exclude_packages:
-                self.exclude_packages.add(pkg)
+            self.exclude_packages.add(pkg)
 
     def build_distribution(self):
         """Create the distribution tree.
@@ -134,7 +160,7 @@ class BuilderApplication(Application):
         This method does everything needed to knit a distribution
         together; it should be refactored substantially.
         """
-        dep_sources = {} 
+        dep_sources = {}
         top = self.get_component(self.resource, self.resource_url)
         top.write_package(self.destination)
         distclass = self.options.distribution_class
@@ -280,23 +306,15 @@ class BuilderApplication(Application):
             # have the package as a side effect of something else
             return
         source = None
-        if name in self.locations:
-            url = self.locations[name]
-        else:
-            url = fallback
-            if not url:
-                self.logger.warning("resource %s not configured;"
-                                    " no fallback URL" % name)
-                return
-            self.logger.info("resource %s not configured;"
-                             " using fallback URL" % name)
+        url = fallback or self.locations.get(name)
+        if not url:
+            self.logger.warning("resource %s not configured;"
+                                " no location known" % name)
+            return
         if source is None:
             self.logger.debug("loading resource '%s' from %s",
                               name, url)
             source = self.loader.load_mutable_copy(url)
-            tests_dir = os.path.join(source, "tests")
-            if os.path.exists(tests_dir):
-                rmtree_force(tests_dir)
 
         self.ip.copyTree(source, destination)
 
@@ -307,10 +325,24 @@ class BuilderApplication(Application):
         """
         pwd = os.getcwd()
         os.chdir(self.tmpdir)
-        cmdline = ("tar", "czf", self.target_file, self.target_name)
-        runlog.report_command(" ".join(cmdline))
+        cmdline1 = ("tar", "czf", self.target_file,
+                    "--owner=0", "--group=0", "--mode=a+r",
+                    self.target_name)
+        cmdline2 = ("tar", "czf", self.target_file, self.target_name)
+        # The os.spawn?p*() functions are not available on Windows, so
+        # we need to search the PATH ourselves:
+        cmdpath = runlog.find_command(cmdline1[0])
+        runlog.report_command(" ".join(cmdline1))
         try:
-            rc = os.spawnlp(os.P_WAIT, cmdline[0], *cmdline)
+            rc = os.spawnv(os.P_WAIT, cmdpath, cmdline1)
+            if os.WIFEXITED(rc) and os.WEXITSTATUS(rc) > 0:
+                # didn't like the command line; maybe not GNU tar?
+                if os.path.exists(self.target_file):
+                    os.unlink(self.target_file)
+                    cmdpath = runlog.find_command(cmdline2[0])
+                runlog.report_exit_code(rc)
+                runlog.report_command(" ".join(cmdline2))
+                rc = os.spawnv(os.P_WAIT, cmdpath, cmdline2)
         finally:
             os.chdir(pwd)
         runlog.report_exit_code(rc)
@@ -344,7 +376,8 @@ class BuilderApplication(Application):
             except zpkgtools.LoadingError, e:
                 self.error(str(e), e.exitcode)
             self.write_manifests()
-            self.create_tarball()
+            if not self.options.tree_only:
+                self.create_tarball()
             self.cleanup()
         except:
             print >>sys.stderr, "----\ntemporary files are in", self.tmpdir
@@ -361,7 +394,7 @@ class Component:
         self.pkginfo = None
         self.pubinfo = None
         self.source = self.ip.loader.load(self.url)
-        specs = include.load(self.source, url=self.url)
+        specs = include.load(self.source)
         if specs.loads:
             source = self.ip.loader.load_mutable_copy(self.url)
             if source != self.source:
@@ -369,7 +402,7 @@ class Component:
                 # we need to re-load the specs to get the .source
                 # attribute of the specification objects correct
                 # XXX need test!
-                specs = include.load(source, url=self.url)
+                specs = include.load(source)
             self.ip.addIncludes(self.source, specs.loads)
         specs.collection.cook()
         specs.distribution.cook()
@@ -378,8 +411,7 @@ class Component:
         #
         # Check that this package is valid:
         #
-        setup_cfg = os.path.join(self.source, package.PACKAGE_CONF)
-        if self.is_python_package() or os.path.isfile(setup_cfg):
+        if self.is_python_package() or self.has_packaging_data():
             return
         raise zpkgtools.Error(
             "%r is an invalid distribution component: all components must"
@@ -440,6 +472,17 @@ class Component:
             dir = self.source
         return os.path.isfile(os.path.join(dir, "__init__.py"))
 
+    def has_packaging_data(self):
+        """Return True iff this component contains packaging metadata."""
+        # Should PUBLICATION.cfg count toword this?
+        dir = self.source
+        for fn in (package.PACKAGE_CONF,
+                   include.PACKAGE_CONF,
+                   "DEPENDENCIES.cfg"):
+            if os.path.isfile(os.path.join(dir, fn)):
+                return True
+        return False
+
     def write_package(self, destination):
         self.destination = destination
         if not os.path.exists(destination):
@@ -597,6 +640,9 @@ def parse_args(argv):
     parser.add_option(
         "-s", dest="include_support_code", action="store_true",
         help="include copies of the zpkgtools support code (the default)")
+    parser.add_option(
+        "-t", "--tree", dest="tree_only", action="store_true",
+        help="generate an unpacked distribution tree, not a tarball")
     parser.add_option(
         "-v", dest="version",
         help="version label for the new distribution")
@@ -614,11 +660,21 @@ def parse_args(argv):
               " (dependencies will be ignored)"))
 
     options, args = parser.parse_args(argv[1:])
-    if len(args) != 1:
-        parser.error("wrong number of arguments")
+    if len(args) > 1:
+        parser.error("too many arguments")
+    try:
+        for exclusion in options.exclude_packages:
+            config.resource_name(exclusion)
+    except ValueError, e:
+        parser.error(str(e))
     options.program = prog
     options.args = args
-    options.resource = args[0]
+    options.support_packages = [
+        (pkgname, None) for pkgname in options.support_packages]
+    if args:
+        options.resource = args[0]
+    else:
+        options.resource = None
     if options.revision_tag and not options.version:
         options.version = version_from_tagname(options.revision_tag)
     if not options.version:
@@ -640,8 +696,11 @@ def main(argv=None):
     try:
         options = parse_args(argv)
     except SystemExit, e:
-        print >>sys.stderr, e
-        return 2
+        if e.code is None or isinstance(e.code, int):
+            return 0
+        else:
+            print >>sys.stderr, e
+            return 2
 
     try:
         app = BuilderApplication(options)
